@@ -6,16 +6,8 @@
 #include <math.h>
 #include <time.h>
 #include <string.h>
-
-#if defined(__STDC__)
 #include <stdlib.h>
-#endif
-
-#ifdef _POSIX_SOURCE
 #include <unistd.h>
-#else
-extern int close();
-#endif
 
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
@@ -29,61 +21,33 @@ extern int close();
 #include <Xm/TextF.h>
 #include <Xm/Separator.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
-#include "preferences.h"
-#include "patchlevel.h"
-#include "net.h"
-#include "ps.h"
+#include "xephem.h"
 
-extern Widget toplevel_w;
-extern XtAppContext xe_app;
-extern Colormap xe_cm;
-
-extern Now *mm_get_now P_((void));
-extern char *getShareDir P_((void));
-extern char *syserrstr P_((void));
-extern int gif2pm P_((Display *dsp, Colormap cm, unsigned char gif[], int ngif,
-    int *wp, int *hp, Pixmap *pmp, char why[]));
-extern void get_something P_((Widget w, char *resource, XtArgVal value));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void pm_set P_((int percentage));
-extern void pm_down P_((void));
-extern void pm_up P_((void));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-extern void watch_cursor P_((int want));
-extern void wtip P_((Widget w, char *tip));
-extern void xe_msg P_((char *msg, int app_modal));
-
-#define	LISTROWS	6	/* rows in list */
+#define	LISTROWS	7	/* rows in list */
 #define	DESCOLS		10	/* columns in avdb for designation */
 #define	NAMCOLS		20	/* longest selection name */
 #define	TFCOL		12	/* columns in text fields */
 #define	AVEPOCH		1900	/* epoch of entries in avdb */
 static char avdb[] = "starcensus.txt";		/* AAVSO star list */
-static char avhost[] = "charts.aavso.org";	/* host */
-static char avcgi[] = "/cgi-bin/xephemstar.pl";	/* aavso's cgi-bin script */
+static char avhost[] = "www.aavso.org";		/* host */
+static char avcgi[] = "/cgi-bin/kstar.pl";	/* aavso's cgi-bin script */
 static char default_str[] = "default";		/* knows to use nice interval */
 static char latest_str[] = "latest";		/* knows to use latest */
 
-static void av_create P_((void));
-static void av_help_cb P_((Widget w, XtPointer client, XtPointer call));
-static void av_close_cb P_((Widget w, XtPointer client, XtPointer call));
-static void av_print_cb P_((Widget w, XtPointer client, XtPointer call));
-static void av_list_cb P_((Widget w, XtPointer client, XtPointer call));
-static void av_go_cb P_((Widget w, XtPointer client, XtPointer call));
-static void av_name_cb P_((Widget w, XtPointer client, XtPointer call));
-static void mkPrompt P_((Widget parent, char *prompt, Widget *text_wp,
-    char *name));
-static void mkTop P_((Widget rc_w));
-static FILE * openDB P_((void));
-static int fetchpm P_((void));
-static void cleanLine P_((char *line));
-static void fetchAndShow P_((void));
-static void av_print P_((void));
+static void av_create (void);
+static void av_help_cb (Widget w, XtPointer client, XtPointer call);
+static void av_close_cb (Widget w, XtPointer client, XtPointer call);
+static void av_print_cb (Widget w, XtPointer client, XtPointer call);
+static void av_list_cb (Widget w, XtPointer client, XtPointer call);
+static void av_go_cb (Widget w, XtPointer client, XtPointer call);
+static void av_name_cb (Widget w, XtPointer client, XtPointer call);
+static void mkPrompt (Widget parent, char *prompt, Widget *text_wp, char *name);
+static void mkTop (Widget rc_w);
+static FILE * openDB (void);
+static int fetchpm (void);
+static void cleanLine (char *line);
+static void fetchAndShow (void);
+static void av_print (void);
 
 static Widget av_w;		/* main shell */
 static Widget avgrl_w;		/* Label for displaying graph */
@@ -92,8 +56,28 @@ static Widget avname_w;		/* TF for AAVSO star name */
 static Widget avdes_w;		/* TF for AAVSO star designation */
 static Widget avjd1_w;		/* TF for starting time */
 static Widget avjd2_w;		/* TF for ending time */
+static Widget avfit_w;		/* TF for running average period, days */
 static Pixmap curve_pm;		/* the aavso light curve pixmap */
 
+/* AAVSO binary query options */
+typedef struct {
+    char *name;			/* TB name */
+    char *desc;			/* description */
+    char *tstr;			/* true URL string */
+    char *fstr;			/* false URL string */
+    Widget w;			/* controlling TB */
+} ABOpt;
+/* N.B.: must be in order required by URL */
+static ABOpt abopts[] = {
+    { "FainterThan", "Fainter-than data", "yes", "no"},
+    { "CCDV",        "CCDV data",         "yes", "no"},
+    { "CCDI",        "CCDI data",         "yes", "no"},
+    { "CCDR",        "CCDR data",         "yes", "no"},
+    { "CCDB",        "CCDB data",         "yes", "no"},
+    { "Visual",      "Visual data",       "yes", "no"},
+    { "Discrepant",  "Discrepant data",   "yes", "no"},
+};
+
 static char aavsocategory[] = "AAVSO";	/* Save category */
 
 /* create and bring up the AAVSO dialog */
@@ -210,16 +194,13 @@ static FILE *
 openDB()
 {
 	char fn[1024];
-	char buf[1024];
 	FILE *fp;
 
 	/* open */
 	(void) sprintf (fn, "%s/auxil/%s", getShareDir(), avdb);
 	fp = fopen (fn, "r");
-	if (!fp) {
-	    sprintf (buf, "%s:\n%s\n", fn, syserrstr());
-	    xe_msg (buf, 1);
-	}
+	if (!fp)
+	    xe_msg (1, "%s:\n%s\n", fn, syserrstr());
 	return (fp);
 }
 
@@ -257,6 +238,7 @@ av_create()
 	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
 	av_w = XtCreatePopupShell ("AAVSO", topLevelShellWidgetClass,
 							toplevel_w, args, n);
+	setup_icon (av_w);
 	set_something (av_w, XmNcolormap, (XtArgVal)xe_cm);
 	sr_reg (av_w, "XEphem*AAVSO.x", aavsocategory, 0);
 	sr_reg (av_w, "XEphem*AAVSO.y", aavsocategory, 0);
@@ -356,10 +338,10 @@ static void
 mkTop (rc_w)
 Widget rc_w;
 {
-	Widget lrc_w, rrc_w;
+	Widget lrc_w, rrc_w, orc_w;
 	Widget f_w, w;
 	Arg args[20];
-	int n;
+	int i, n;
 
 	/* 2 vertical RCs placed adjacent in a form */
 	n = 0;
@@ -387,7 +369,7 @@ Widget rc_w;
 	/* title to balance with right side */
 	n = 0;
 	w = XmCreateLabel (lrc_w, "LL", args, n);
-	set_xmstring (w, XmNlabelString, "Name and date range:");
+	set_xmstring (w, XmNlabelString, "Setup:");
 	XtManageChild (w);
 
 	/* designation */
@@ -408,12 +390,16 @@ Widget rc_w;
 	wtip (avjd2_w, "Enter ending JD, or Date, or \"latest\"");
 	XtAddCallback (avjd2_w, XmNactivateCallback, av_go_cb, NULL);
 
+	/* running average */
+	mkPrompt (lrc_w, "Smooth fit, days:", &avfit_w, "SmoothDays");
+	wtip (avfit_w, "Running average period, or 0 to disable");
+
 	/* stuff in right column */
 
 	n = 0;
 	w = XmCreateLabel (rrc_w, "L", args, n);
 	XtManageChild (w);
-	set_xmstring (w, XmNlabelString, "Near last Sky View pick:"); 
+	set_xmstring (w, XmNlabelString, "AAVSO Stars near last Sky View pick:"); 
 
 	n = 0;
 	XtSetArg (args[n], XmNvisibleItemCount, LISTROWS); n++;
@@ -423,6 +409,33 @@ Widget rc_w;
 	XtAddCallback (avlist_w, XmNdefaultActionCallback, av_list_cb, NULL);
 	wtip (avlist_w, "Stars nearest to last Sky View AAVSO pick");
 	XtManageChild(avlist_w);
+
+	/* binary options in RC below */
+
+	n = 0;
+	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
+	w = XmCreateLabel (rc_w, "AOL", args, n);
+	set_xmstring (w, XmNlabelString, "Plot:");
+	XtManageChild (w);
+
+	n = 0;
+	XtSetArg (args[n], XmNpacking, XmPACK_COLUMN); n++;
+	XtSetArg (args[n], XmNmarginWidth, 5); n++;
+	XtSetArg (args[n], XmNspacing, 5); n++;
+	XtSetArg (args[n], XmNnumColumns, 4); n++;
+	orc_w = XmCreateRowColumn (rc_w, "Plot", args, n);
+	XtManageChild (orc_w);
+
+	for (i = 0; i < XtNumber(abopts); i++) {
+	    ABOpt *ap = &abopts[i];
+
+	    n = 0;
+	    w = XmCreateToggleButton (orc_w, ap->name, args, n);
+	    set_xmstring (w, XmNlabelString, ap->desc);
+	    XtManageChild (w);
+	    sr_reg (w, NULL, aavsocategory, 1);
+	    ap->w = w;
+	}
 }
 
 /* callback from the Close PB */
@@ -447,7 +460,7 @@ XtPointer call;
 	if (curve_pm)
 	    XPSAsk ("AAVSO", av_print);
 	else
-	    xe_msg ("No light curve to print", 1);
+	    xe_msg (1, "No light curve to print");
 }
 
 /* generate a postscript file containing the curve_pm.
@@ -468,7 +481,7 @@ av_print()
 	pw = (int)(72*6.5*wid/hei+.5);	/* width on paper when 6.5 hi */
 
 	XPSXBegin (curve_pm, 0, 0, wid, hei, (int)((8.5*72-pw)/2), 10*72, pw);
-	XPSPixmap (curve_pm, wid, hei, xe_cm, 0);
+	XPSPixmap (curve_pm, wid, hei, xe_cm, 0, 1);
 	XPSXEnd();
 	XPSClose();
 
@@ -578,8 +591,7 @@ XtPointer call;
 	}
 	fclose(fp);
 
-	sprintf (buf, "Can not find '%s' in %s", nameuc, avdb);
-	xe_msg (buf, 1);
+	xe_msg (1, "Can not find '%s' in %s", nameuc, avdb);
 }
 
 /* create a prompt/text field pair */
@@ -619,6 +631,7 @@ char *txtname;
 	XtSetArg (args[n], XmNmaxLength, NAMCOLS); n++;
 	XtSetArg (args[n], XmNcolumns, TFCOL); n++;
 	t_w = XmCreateTextField (f_w, txtname, args, n);
+	sr_reg (t_w, NULL, aavsocategory, 0);
 	XtManageChild(t_w);
 
 	*text_wp = t_w;
@@ -667,19 +680,22 @@ char *def;
 static int
 fetchpm()
 {
-	static char fmt[] = "%s?%s?%s?%s HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n";
+	static char fmt[] = "%s?%s?%s?%s?%g?%s?%s?%s?%s?%s?%s?%s?XEphem HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n";
 	Display *dsp = XtDisplay(toplevel_w);
 	int nrawgif;
 	char buf[1024];
 	char jd1[32], jd2[32];
 	char *idp, id[NAMCOLS+1];
+	char *abop[XtNumber(abopts)];
+	double fit;
 	char *str;
 	int isgif;
 	int length;
 	int w, h;
 	int nr;
 	int fd;
-	unsigned char rawgif[200000];
+	int i;
+	unsigned char rawgif[250000];
 
 	/* announce net activity and give user a way to stop */
 	watch_cursor(1);
@@ -704,12 +720,25 @@ fetchpm()
 	for (str = id; (str = strchr (str, ' ')) != NULL; str++)
 	    *str = '+';
 
+	/* get running average period */
+	str = XmTextFieldGetString (avfit_w);
+	fit = strtod (str, NULL);
+	XtFree (str);
+
+	/* gather options */
+	for (i = 0; i < XtNumber(abopts); i++) {
+	    ABOpt *ap = &abopts[i];
+	    int set = XmToggleButtonGetState (ap->w);
+	    abop[i] = set ? ap->tstr : ap->fstr;
+	}
+
 	/* query server */
-	(void) sprintf (buf, fmt, avcgi, jd1, jd2, id, PATCHLEVEL);
-	xe_msg (buf, 0);
+	(void) sprintf (buf, fmt, avcgi, jd1, jd2, id, fit, abop[0],
+					    abop[1], abop[2], abop[3], abop[4],
+					    abop[5], abop[6], PATCHLEVEL);
 	fd = httpGET (avhost, buf, buf);
 	if (fd < 0) {
-	    xe_msg (buf, 1);
+	    xe_msg (1, "Query: %s", buf);
 	    stopd_down();
 	    watch_cursor(0);
 	    return (-1);
@@ -719,32 +748,33 @@ fetchpm()
 	isgif = 0;
 	length = 0;
 	while (recvline (fd, buf, sizeof(buf)) > 1) {
-	    xe_msg (buf, 0);
-	    if (strstr (buf, "image/gif"))
+	    xe_msg (0, "Rcv: %s", buf);
+	    if (strstr (buf, "Content-Type:") && strstr (buf, "image/gif"))
 		isgif = 1;
-	    if (strstr (buf, "Content-Length"))
+	    if (strstr (buf, "Content-Length:"))
 		length = atoi (buf+15);
 	}
 	if (!isgif) {
 	    while (recvline (fd, buf, sizeof(buf)) > 0)
-		xe_msg (buf, 0);
-	    xe_msg ("Error talking to AAVSO .. see File->Messages\n", 1);
+		xe_msg (0, "Rcv: %s", buf);
+	    xe_msg (1, "Error talking to AAVSO .. see File->System log\n");
 	    close (fd);
 	    stopd_down();
 	    watch_cursor(0);
 	    return (-1);
 	}
-	if (length == 0)
-	    length = 20000;	/* ?? */
+	if (length == 0) {
+	    xe_msg (0, "No Content-Length in header");
+	    length = sizeof(rawgif);	/* ?? */
+	}
 
 	/* remainder should be gif, read into rawgif[] */
 	pm_up();
 	for (nrawgif = 0; nrawgif < sizeof(rawgif); nrawgif += nr) {
 	    pm_set (100*nrawgif/length);
-	    nr = readbytes (fd, rawgif+nrawgif, 4096);
+	    nr = readbytes (fd, rawgif+nrawgif, 2048);
 	    if (nr < 0) {
-		(void) sprintf (buf, "%s:\n%s", avhost, syserrstr());
-		xe_msg (buf, 1);
+		xe_msg (1, "%s:\n%s", avhost, syserrstr());
 		pm_down();
 		close (fd);
 		stopd_down();
@@ -758,14 +788,14 @@ fetchpm()
 	pm_down();
 	close (fd);
 	if (nr > 0) {
-	    xe_msg ("File too large", 1);
+	    xe_msg (1, "File too large");
 	    watch_cursor(0);
 	    return (-1);
 	}
 
 	/* explode into pm */
 	if (gif2pm (dsp, xe_cm, rawgif, nrawgif, &w, &h, &curve_pm, buf) < 0) {
-	    xe_msg (buf, 1);
+	    xe_msg (1, "Gif error:\n%s", buf);
 	    watch_cursor(0);
 	    return (-1);
 	}
@@ -818,4 +848,4 @@ fetchAndShow()
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: aavso.c,v $ $Date: 2000/10/03 21:40:04 $ $Revision: 1.13 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: aavso.c,v $ $Date: 2004/03/16 18:44:38 $ $Revision: 1.20 $ $Name:  $"};
