@@ -2,75 +2,39 @@
 
 #include <stdio.h>
 #include <math.h>
-#include <signal.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <netdb.h>
-
-#ifdef __STDC__
 #include <stdlib.h>
 #include <string.h>
-typedef const void *qsort_arg; 
-#else
-extern char *getenv();
-extern void *memset();
-typedef void *qsort_arg;
-#endif
-
-#if defined(_POSIX_SOURCE)
 #include <unistd.h>
-#else
-extern int close();
-#endif
 
 #include <Xm/Form.h>
 #include <Xm/Label.h>
-#include <Xm/MessageB.h>
 #include <Xm/PushB.h>
 #include <Xm/Separator.h>
 #include <Xm/TextF.h>
 #include <Xm/ToggleB.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
-#include "net.h"
-
-extern Widget toplevel_w;
-#define XtD     XtDisplay(toplevel_w)
-extern Colormap xe_cm;
-extern XtAppContext xe_app;
-
+#include "xephem.h"
 
 #define	TOUT	180		/* max secs to wait for socket data.
 				 * default http timeout is 3 minutes.
 				 */
 
-extern char *syserrstr P_((void));
-extern int isUp P_((Widget w));
-extern void XCheck P_((XtAppContext app));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-extern void watch_cursor P_((int want));
-extern void wtip P_((Widget w, char *tip));
-extern void xe_msg P_((char *msg, int app_modal));
 
-static int net_save P_((void));
-static void net_create_form P_((void));
-static void defaultSOCKS P_((void));
-static void net_setup P_((void));
-static void ok_cb P_((Widget w, XtPointer client, XtPointer call));
-static void tb_cb P_((Widget w, XtPointer client, XtPointer call));
-static void cancel_cb P_((Widget w, XtPointer client, XtPointer call));
-static void help_cb P_((Widget w, XtPointer client, XtPointer call));
-static int tout P_((int maxt, int fd, int w));
-static char *herr P_((char *errmsg));
-static int connect_to P_((int sockfd, struct sockaddr *serv_addr, int addrlen));
-static int stopd_check P_((void));
-static void stopd_cb P_((Widget w, XtPointer data, XtPointer call));
+static int net_save (void);
+static void net_create_form (void);
+static void defaultSOCKS (void);
+static void net_setup (void);
+static void ok_cb (Widget w, XtPointer client, XtPointer call);
+static void tb_cb (Widget w, XtPointer client, XtPointer call);
+static void cancel_cb (Widget w, XtPointer client, XtPointer call);
+static void pw_cb (Widget w, XtPointer client, XtPointer call);
+static void help_cb (Widget w, XtPointer client, XtPointer call);
+static int tout (int maxt, int fd, int w);
+static char *herr (char *errmsg);
+static int connect_to (int sockfd, struct sockaddr *serv_addr, int addrlen);
 
 static Widget netshell_w;	/* the main form dialog */
 static Widget ndir_w;		/* net direct TB */
@@ -78,20 +42,26 @@ static Widget socks_w;		/* SOCKS option TB */
 static Widget socksh_w, socksp_w;/* SOCKS host and port TF */
 static Widget proxy_w;		/* Proxy option TB */
 static Widget proxyh_w, proxyp_w;/* Proxy host and port TF */
-static Widget stopd_w;		/* user's stop dialog */
-static int stopd_stopped;	/* set by the stopd PB */
+static Widget auth_w;		/* Auth option TB */
+static Widget authn_w, authpw_w;/* Auth name and PW TF */
 
 /* passed to the TB callbacks to choose network access method */
 typedef enum {
     NETDIRTB, NETPROXTB, NETSOCKSTB
 } TB;
 
+/* current info */
 static int proxy_on;		/* whether proxy network connection is on */
 static char *proxy_host;	/* proxy host */
 static int proxy_port;		/* proxy port */
 static int socks_on;		/* whether SOCKS network connection is on */
 static char *socks_host;	/* SOCKS host */
 static char *socks_port;	/* SOCKS port (as a string) */
+static int auth_on;		/* whether to add Auth info */
+static char *auth_name;		/* Auth name */
+
+/* we keep the plaintext password here, but just display splats */
+static char *auth_pw;		/* malloced plain text password */
 
 /* buffer for recvlineb() */
 static char rb_linebuf[512];	/* [next .. bad-1] are good */
@@ -123,16 +93,91 @@ net_manage()
 	set_something (netshell_w, XmNiconic, (XtArgVal)False);
 }
 
+/* Base64 stuff lifted from downloader */
+
+static void
+three_to_four(what, where)
+unsigned char *what, *where;
+{
+	static char Table64[64] = {
+	    'A','B','C','D','E','F','G','H',
+	    'I','J','K','L','M','N','O','P',
+	    'Q','R','S','T','U','V','W','X',
+	    'Y','Z','a','b','c','d','e','f',
+	    'g','h','i','j','k','l','m','n',
+	    'o','p','q','r','s','t','u','v',
+	    'w','x','y','z','0','1','2','3',
+	    '4','5','6','7','8','9','+','/'
+	};
+	int i;
+
+	*where=(*what >> 2) & 63;
+	*(where+1)=((*what << 4) | (*(what+1) >> 4)) & 63;
+	*(where+2)=((*(what+1) << 2) | (*(what+2) >> 6)) & 63;
+	*(where+3)=*(what+2) & 63;
+	for (i=0;i<4;i++)
+	    where[i]= Table64[where[i]];
+}
+
+static void
+string_to_base64(plain, b64)
+char *plain, *b64;
+{
+	unsigned char four[4];
+	int len=strlen(plain),len2=0;
+	int i;
+
+	if (len%3) len2=(len/3 +1)*4 +1;
+	else len2=(len/3)*4 +1;
+	while (len>=3) {
+	    three_to_four((unsigned char *)plain,four);
+	    for (i=0;i<4;i++)
+		*(b64++)=four[i];
+	    len-=3;
+	    plain+=3;
+	}
+	if (len) {
+	    unsigned char three[3]={0,0,0};
+	    for (i=0;i<len;i++) {
+		three[i]=*((unsigned char*)plain);
+		plain++;
+	    }
+	    three_to_four(three,four);
+	    for (i=3-(len==1);i<4;i++)
+		four[i]='=';
+	    for (i=0;i<4;i++)
+		*(b64++)=four[i];
+	}
+	*b64=0;
+}
+
+/* add proxy-authorization to buf.
+ * N.B. we assume buf[] ends with \r\n
+ */
+static void
+addAuth (buf)
+char *buf;
+{
+	char *nm = XmTextFieldGetString (authn_w);
+	char plainbuf[1024], b64buf[1024];
+
+	sprintf (plainbuf, "%s:%s", nm, auth_pw);
+	string_to_base64 (plainbuf, b64buf);
+	XtFree (nm);
+
+	/* insert before last \r\n */
+	sprintf (buf+strlen(buf)-2, "Proxy-Authorization: Basic %s\r\n\r\n",
+								    b64buf);
+}
+
 /* open the host, do the given GET cmd, and return a socket fd for the result.
  * return -1 and with excuse in msg[], else 0 if ok.
+ * N.B. can be called before we are created if net set in app defaults.
  */
 int
-httpGET (host, GETcmd, msg)
-char *host;
-char *GETcmd;
-char msg[];
+httpGET (char *host, char *GETcmd, char msg[])
 {
-	char buf[1024];
+	char buf[2048];
 	int fd;
 	int n;
 
@@ -141,18 +186,24 @@ char msg[];
 	    fd = mkconnection (proxy_host, proxy_port, msg);
 	    if (fd < 0)
 		return (-1);
-	    (void) sprintf (buf, "GET http://%s", host);
-	    if (GETcmd[0] != '/')
-		(void) strcat (buf, "/");
-	    (void) strcat (buf, GETcmd);
 	} else {
 	    /* SOCKS or direct are both handled by mkconnection() */
 	    fd = mkconnection (host, 80, msg);
 	    if (fd < 0)
 		return (-1);
-	    (void) sprintf (buf, "GET %s", GETcmd);
 	}
 
+	/* use absoluteURL in lieu of HTTP1.1 for now */
+	(void) sprintf (buf, "GET http://%s%s", host, GETcmd);
+
+	/* add proxy auth if enabled */
+	if (!auth_w)
+	    net_create_form();
+	if (XmToggleButtonGetState (auth_w))
+	    addAuth(buf);
+
+	xe_msg (0, "http: %s", buf);
+
 	/* send it */
 	n = strlen (buf);
 	if (sendbytes(fd, (unsigned char *)buf, n) < 0) {
@@ -169,21 +220,19 @@ char msg[];
  * if ok return file descriptor, else -1 with excuse in msg[].
  * try Socks if see socks_on.
  * reset recvlineb() readahead in case old stuff left behind.
+ * N.B. we assume SIGPIPE is SIG_IGN
  */
 int
-mkconnection (host, port, msg)
-char *host;	/* name of server */
-int port;	/* TCP port */
-char msg[];	/* return diagnostic message here, if returning -1 */
+mkconnection (
+char *host,	/* name of server */
+int port,	/* TCP port */
+char msg[])	/* return diagnostic message here, if returning -1 */
 {
 
 	struct sockaddr_in serv_addr;
 	struct hostent  *hp;
 	int sockfd;
 
-	/* don't want signal if loose connection to server */
-	(void) signal (SIGPIPE, SIG_IGN);
-
 	/* lookup host address.
 	 * TODO: time out but even SIGALRM doesn't awaken this if it's stuck.
 	 *   I bet that's why netscape forks a separate dnshelper process!
@@ -291,10 +340,7 @@ char msg[];	/* return diagnostic message here, if returning -1 */
  * return 0 if ok else -1
  */
 int
-sendbytes (fd, buf, n)
-int fd;
-unsigned char buf[];
-int n;
+sendbytes (int fd, unsigned char buf[], int n)
 {
 	int ns, tot;
 
@@ -309,13 +355,10 @@ int n;
 }
 
 /* receive exactly n bytes from socket fd into buf.
- * return n if ok else -1
+ * return -1, 0 or n.
  */
 int
-recvbytes (fd, buf, n)
-int fd;
-unsigned char buf[];
-int n;
+recvbytes (int fd, unsigned char buf[], int n)
 {
 	int ns, tot;
 
@@ -324,7 +367,7 @@ int n;
 		return (-1);
 	    ns = read (fd, (void *)(buf+tot), n-tot);
 	    if (ns <= 0)
-		return (-1);
+		return (ns);
 	}
 	return (n);
 }
@@ -334,10 +377,7 @@ int n;
  * return count, or 0 on eof or -1 on error.
  */
 int
-readbytes (fd, buf, n)
-int fd;
-unsigned char buf[];
-int n;
+readbytes (int fd, unsigned char buf[], int n)
 {
 	int ns;
 
@@ -353,10 +393,7 @@ int n;
  * N.B. this never reads ahead -- if that's ok, recvlineb() is better
  */
 int
-recvline (fd, buf, max)
-int fd;
-char buf[];
-int max;
+recvline (int fd, char buf[], int max)
 {
 	unsigned char c;
 	int n;
@@ -382,10 +419,7 @@ int max;
  * N.B. we read ahead and can hide bytes after each call.
  */
 int
-recvlineb (sock, buf, size)
-int sock;
-char *buf;
-int size;
+recvlineb (int sock, char *buf, int size)
 {
 	char *origbuf = buf;		/* save to prevent overfilling buf */
 	char c = '\0';
@@ -424,49 +458,6 @@ int size;
 	return (nr);
 }
 
-/* pop up a modal dialog to allow aborting the pixel reading */
-void
-stopd_up()
-{
-	/* create if first time */
-	if (!stopd_w) {
-	    Arg args[20];
-	    int n;
-
-	    n = 0;
-	    XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL);  n++;
-	    XtSetArg(args[n], XmNtitle, "xephem Net Stop");  n++;
-	    stopd_w = XmCreateInformationDialog(toplevel_w, "NetStop", args, n);
-	    set_something (stopd_w, XmNcolormap, (XtArgVal)xe_cm);
-	    XtAddCallback (stopd_w, XmNokCallback, stopd_cb, (XtPointer)0);
-	    XtAddCallback (stopd_w, XmNmapCallback, prompt_map_cb, NULL);
-	    XtUnmanageChild(XmMessageBoxGetChild(stopd_w,
-						    XmDIALOG_CANCEL_BUTTON));
-	    XtUnmanageChild(XmMessageBoxGetChild(stopd_w,XmDIALOG_HELP_BUTTON));
-	    set_xmstring(stopd_w, XmNmessageString, "Press Stop to cancel...");
-	    set_xmstring(stopd_w, XmNokLabelString, "Stop");
-	}
-
-	/* get it up */
-	XtManageChild (stopd_w);
-	XFlush (XtD);
-	XmUpdateDisplay (toplevel_w);
-	XSync (XtD, 0);
-
-	/* init button not pressed */
-	stopd_stopped = 0;
-}
-
-/* bring down the user stop dialog */
-void
-stopd_down()
-{
-	if (stopd_w)
-	    XtUnmanageChild (stopd_w);
-	stopd_stopped = 0;
-}
-
 static void
 net_create_form()
 {
@@ -483,6 +474,7 @@ net_create_form()
 	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
 	netshell_w = XtCreatePopupShell ("NetSetup", topLevelShellWidgetClass,
 							toplevel_w, args, n);
+	setup_icon (netshell_w);
 	set_something (netshell_w, XmNcolormap, (XtArgVal)xe_cm);
 	sr_reg (netshell_w, "XEphem*NetSetup.x", netcategory, 0);
 	sr_reg (netshell_w, "XEphem*NetSetup.y", netcategory, 0);
@@ -602,11 +594,62 @@ net_create_form()
 	    XtManageChild (proxyh_w);
 	    sr_reg (proxyh_w, NULL, netcategory, 1);
 
+	    /* make the Auth option and info */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNtopWidget, proxyh_w); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNindicatorType, XmN_OF_MANY); n++;
+	    auth_w = XmCreateToggleButton (netform_w, "Auth", args, n);
+	    set_xmstring (auth_w, XmNlabelString, "Auth Name");
+	    wtip (auth_w, "Apply authentication name and password");
+	    XtManageChild (auth_w);
+	    sr_reg (auth_w, NULL, netcategory, 1);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNtopWidget, proxyh_w); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNleftWidget, auth_w); n++;
+	    XtSetArg (args[n], XmNleftOffset, 10); n++;
+	    XtSetArg (args[n], XmNcolumns, 15); n++;
+	    authn_w = XmCreateTextField (netform_w, "AuthName", args, n);
+	    wtip (authn_w, "Authentication name");
+	    XtManageChild (authn_w);
+	    sr_reg (authn_w, NULL, netcategory, 1);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNtopWidget, proxyh_w); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNleftWidget, authn_w); n++;
+	    XtSetArg (args[n], XmNleftOffset, 10); n++;
+	    w = XmCreateLabel (netform_w, "Password", args, n);
+	    XtManageChild (w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNtopWidget, proxyh_w); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNleftWidget, w); n++;
+	    XtSetArg (args[n], XmNleftOffset, 10); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNcolumns, 15); n++;
+	    authpw_w = XmCreateTextField (netform_w, "AuthPassword", args, n);
+	    XtAddCallback (authpw_w, XmNmodifyVerifyCallback, pw_cb, 0);
+	    wtip (authpw_w, "Authentication password");
+	    XtManageChild (authpw_w);
+
+	    /* malloc null string to start */
+	    auth_pw = XtNewString ("");
+
+
 	/* make the controls across the bottom under a separator */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, proxyh_w); n++;
+	XtSetArg (args[n], XmNtopWidget, authpw_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	w = XmCreateSeparator (netform_w, "Sep", args, n);
@@ -691,6 +734,10 @@ net_setup ()
 	    XmTextFieldSetString (socksh_w, socks_host);
 	if (socks_port)
 	    XmTextFieldSetString (socksp_w, socks_port);
+
+	XmToggleButtonSetState (auth_w, auth_on, False);
+	if (auth_name)
+	    XmTextFieldSetString (authn_w, auth_name);
 }
 
 /* save the dialog as our static state.
@@ -718,7 +765,7 @@ net_save ()
 	if (proxy_on) {
 	    fd = mkconnection (proxy_host, proxy_port, msg);
 	    if (fd < 0) {
-		xe_msg (msg, 1);
+		xe_msg (1, "%s", msg);
 		proxy_on = 0;
 		net_setup ();
 		allok = 0;
@@ -736,13 +783,18 @@ net_save ()
 	    /* TODO: how to test? */
 	    fd = mkconnection (socks_host, atoi(socks_port), msg);
 	    if (fd < 0) {
-		xe_msg (msg, 1);
+		xe_msg (1, "%s", msg);
 		socks_on = 0;
 		net_setup ();
 		allok = 0;
 	    } else
 		(void) close (fd);
 	}
+	auth_on = XmToggleButtonGetState (auth_w);
+	if (auth_name)
+	    XtFree (auth_name);
+	auth_name = XmTextFieldGetString (authn_w);
+	/* TODO: how to test? */
 
 	watch_cursor (0);
 
@@ -773,6 +825,38 @@ XtPointer call;
 	XtPopdown (netshell_w);
 }
 
+/* called whenever the Password text field is edited.
+ * add to auth_pw but echo as *
+ */
+/* ARGSUSED */
+static void
+pw_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	XmTextVerifyCallbackStruct *vp = (XmTextVerifyCallbackStruct *)call;
+	int l = XmTextFieldGetLastPosition(w);
+	int sp = vp->startPos;
+	int ep = vp->endPos;
+
+	if (sp < ep) {
+	    /* cut sp .. ep-1 */
+	    memmove (auth_pw+sp, auth_pw+ep, l-ep+1);		/* EOS too */
+	    l -= ep-sp;
+	}
+
+	if (vp->text->length > 0) {
+	    /* insert 1 @ sp */
+	    auth_pw = XtRealloc (auth_pw, l+2);			/* new + EOS */
+	    memmove (auth_pw+sp+1, auth_pw+sp, l-sp+1);		/* EOS too */
+	    auth_pw[sp] = vp->text->ptr[0];
+
+	    /* echo as '*' */
+	    vp->text->ptr[0] = '*';
+	}
+}
+
 /* called from any of the choice toggle buttons.
  * client is one of the TB enums to tell us which.
  */
@@ -784,11 +868,12 @@ XtPointer client;
 XtPointer call;
 {
 	if (XmToggleButtonGetState(w)) {
-	    switch ((int)client) {
+	    switch ((long int)client) {
 	    case NETDIRTB:
-		/* turn off proxy and socks */
+		/* turn off proxy, socks and auth */
 		XmToggleButtonSetState (proxy_w, False, False);
 		XmToggleButtonSetState (socks_w, False, False);
+		XmToggleButtonSetState (auth_w, False, False);
 		break;
 	    case NETPROXTB:
 		/* turn off direct and socks */
@@ -801,8 +886,8 @@ XtPointer call;
 		XmToggleButtonSetState (proxy_w, False, False);
 		break;
 	    default:
-		printf ("FS: bad client: %d\n", (int)client);
-		exit(1);
+		printf ("FS: bad client: %d\n", (int)(long int)client);
+		abort();
 	    }
 	}
 }
@@ -854,7 +939,7 @@ int w;
 		break;
 	    default:
 		printf ("Bug: tout() called with %d\n", w);
-		exit(1);
+		abort();
 	    }
 
 	    tv.tv_sec = 1;
@@ -941,33 +1026,5 @@ int addrlen;
 	return (0);
 }
 
-/* called when the user presses the net Stop button */
-/* ARGSUSED */
-static void
-stopd_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
-{
-	xe_msg ("User stop", 0);
-	stopd_stopped = 1;
-}
-
-/* poll whether the user wants to stop, then reset flag if do.
- * return -1 to stop, else 0.
- */
-static int
-stopd_check()
-{
-	/* check for user button presses */
-	XCheck (xe_app);
-
-	if (stopd_stopped) {
-	    stopd_stopped = 0;
-	    return (-1);
-	}
-	return (0);
-}
-
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: netmenu.c,v $ $Date: 2000/10/08 08:09:41 $ $Revision: 1.12 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: netmenu.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.28 $ $Name:  $"};
