@@ -8,40 +8,41 @@
 
 #include <stdio.h>
 #include <math.h>
-#if defined(__STDC__)
 #include <stdlib.h>
-#endif
 
-#include "P_.h"
 #include "astro.h"
-#include "circum.h"
 #include "preferences.h"
 
 
-static int obj_planet P_((Now *np, Obj *op));
-static int obj_fixed P_((Now *np, Obj *op));
-static int obj_elliptical P_((Now *np, Obj *op));
-static int obj_hyperbolic P_((Now *np, Obj *op));
-static int obj_parabolic P_((Now *np, Obj *op));
-static int sun_cir P_((Now *np, Obj *op));
-static int moon_cir P_((Now *np, Obj *op));
-static void cir_sky P_((Now *np, double lpd, double psi, double rp, double *rho,
-    double lam, double bet, double lsn, double rsn, Obj *op));
-static void cir_pos P_((Now *np, double bet, double lam, double *rho, Obj *op));
-static void elongation P_((double lam, double bet, double lsn, double *el));
-static void deflect P_((double mjd1, double lpd, double psi, double rsn,
-    double lsn, double rho, double *ra, double *dec));
-static double h_albsize P_((double H));
+static int obj_planet (Now *np, Obj *op);
+static int obj_binary (Now *np, Obj *op);
+static int obj_2binary (Now *np, Obj *op);
+static int obj_fixed (Now *np, Obj *op);
+static int obj_elliptical (Now *np, Obj *op);
+static int obj_hyperbolic (Now *np, Obj *op);
+static int obj_parabolic (Now *np, Obj *op);
+static int sun_cir (Now *np, Obj *op);
+static int moon_cir (Now *np, Obj *op);
+static double solveKepler (double M, double e);
+static void binaryStarOrbit (double t, double T, double e, double o, double O,
+    double i, double a, double P, double *thetap, double *rhop);
+static void cir_sky (Now *np, double lpd, double psi, double rp, double *rho,
+    double lam, double bet, double lsn, double rsn, Obj *op);
+static void cir_pos (Now *np, double bet, double lam, double *rho, Obj *op);
+static void elongation (double lam, double bet, double lsn, double *el);
+static void deflect (double mjd1, double lpd, double psi, double rsn,
+    double lsn, double rho, double *ra, double *dec);
+static double h_albsize (double H);
 
 /* given a Now and an Obj, fill in the approprirate s_* fields within Obj.
  * return 0 if all ok, else -1.
  */
 int
-obj_cir (np, op)
-Now *np;
-Obj *op;
+obj_cir (Now *np, Obj *op)
 {
+	op->o_flags &= ~NOCIRCUM;
 	switch (op->o_type) {
+	case BINARYSTAR: return (obj_binary (np, op));
 	case FIXED:	 return (obj_fixed (np, op));
 	case ELLIPTICAL: return (obj_elliptical (np, op));
 	case HYPERBOLIC: return (obj_hyperbolic (np, op));
@@ -49,16 +50,14 @@ Obj *op;
 	case EARTHSAT:   return (obj_earthsat (np, op));
 	case PLANET:     return (obj_planet (np, op));
 	default:
-	    printf ("obj_cir() called with type %d\n", op->o_type);
-	    exit(1);
+	    printf ("obj_cir() called with type %d %s\n", op->o_type, op->o_name);
+	    abort();
 	    return (-1);	/* just for lint */
 	}
 }
 
 static int
-obj_planet (np, op)
-Now *np;
-Obj *op;
+obj_planet (Now *np, Obj *op)
 {
 	double lsn, rsn;	/* true geoc lng of sun; dist from sn to earth*/
 	double lpd, psi;	/* heliocentric ecliptic long and lat */
@@ -66,24 +65,27 @@ Obj *op;
 	double rho;		/* dist from earth */
 	double lam, bet;	/* geocentric ecliptic long and lat */
 	double dia, mag;	/* angular diameter at 1 AU and magnitude */
-	double f;		/* fractional phase from earth */
-	int p;
+	PLCode p;
 
 	/* validate code and check for a few special cases */
-	p = op->pl.pl_code;
+	p = op->pl_code;
+	if (p == SUN)
+	    return (sun_cir (np, op));
+	if (p == MOON)
+	    return (moon_cir (np, op));
+	if (op->pl_moon != X_PLANET)
+	    return (plmoon_cir (np, op));
 	if (p < 0 || p > MOON) {
 	    printf ("unknown planet code: %d\n", p);
-	    exit(1);
+	    abort();
 	}
-	else if (p == SUN)
-	    return (sun_cir (np, op));
-	else if (p == MOON)
-	    return (moon_cir (np, op));
+
+	/* planet itself */
 
 	/* find solar ecliptical longitude and distance to sun from earth */
 	sunpos (mjed, &lsn, &rsn, 0);
 
-	/* find helio long/lat; sun/planet and earth/plant dist; ecliptic
+	/* find helio long/lat; sun/planet and earth/planet dist; ecliptic
 	 * long/lat; diameter and mag.
 	 */
 	plans(mjed, p, &lpd, &psi, &rp, &rho, &lam, &bet, &dia, &mag);
@@ -91,61 +93,175 @@ Obj *op;
 	/* fill in all of op->s_* stuff except s_size and s_mag */
 	cir_sky (np, lpd, psi, rp, &rho, lam, bet, lsn, rsn, op);
 
-	/* compute magnitude and angular size */
-	f = op->s_phase ? 5*log10(rp*rho) - 5*log10(op->s_phase/100) : 100;
-	set_smag (op, mag+f);
+	/* set magnitude and angular size */
+	set_smag (op, mag);
 	op->s_size = (float)(dia/rho);
 
 	return (0);
 }
 
 static int
-obj_fixed (np, op)
-Now *np;
-Obj *op;
+obj_binary (Now *np, Obj *op)
+{
+	/* always compute circumstances of primary */
+	if (obj_fixed (np, op) < 0)
+	    return (0);
+
+	/* compute secondary only if requested, and always reset request flag */
+	if (!op->b_2compute)
+	    return (0);
+	op->b_2compute = 0;
+	return (obj_2binary (np, op));
+}
+
+/* compute position of secondary component of a BINARYSTAR */
+static int
+obj_2binary (Now *np, Obj *op)
+{
+	if (op->b_nbp > 0) {
+	    /* we just have discrete pa/sep, project each from primary */
+	    int i;
+	    for (i = 0; i < op->b_nbp; i++) {
+		BinPos *bp = &op->b_bp[i];
+		bp->bp_dec = op->s_dec + bp->bp_sep*cos(bp->bp_pa);
+		bp->bp_ra = op->s_ra + bp->bp_sep*sin(bp->bp_pa)/cos(op->s_dec);
+	    }
+	} else {
+	    BinOrbit *bp = &op->b_bo;
+	    double t, theta, rho;
+
+	    mjd_year (mjd, &t);
+	    binaryStarOrbit (t, bp->bo_T, bp->bo_e, bp->bo_o, bp->bo_O,
+				bp->bo_i, bp->bo_a, bp->bo_P, &theta, &rho);
+	    bp->bo_pa = (float)theta;
+	    bp->bo_sep = (float)rho;
+	    rho = degrad(rho/3600.);	/* arc secs to rads */
+	    bp->bo_dec = op->s_dec + rho*cos(theta);
+	    bp->bo_ra =  op->s_ra  + rho*sin(theta)/cos(op->s_dec);
+	}
+
+	return (0);
+}
+
+/* from W. M. Smart */
+static void
+binaryStarOrbit (
+double t,		/* desired ephemeris epoch, year */
+double T,		/* epoch of periastron, year */
+double e,		/* eccentricity */
+double o,		/* argument of periastron, degrees */
+double O,		/* ascending node, degrees */
+double i,		/* inclination, degrees */
+double a,		/* semi major axis, arcsecs */
+double P,		/* period, years */
+double *thetap,		/* position angle, rads E of N */
+double *rhop)		/* separation, arcsecs */
+{
+	double M, E, cosE, nu, cosnu, r, rho, theta;
+
+	/* find mean anomaly, insure 0..2*PI */
+	M = 2*PI/P*(t-T);
+	range (&M, 2*PI);
+
+	/* solve for eccentric anomaly */
+	E = solveKepler (M, e);
+	cosE = cos(E);
+
+	/* find true anomaly and separation */
+	cosnu = (cosE - e)/(1.0 - e*cosE);
+	r = a*(1.0 - e*e)/(1.0 + e*cosnu);
+	nu = acos(cosnu);
+	if (E > PI)
+	    nu = -nu;
+
+	/* project onto sky */
+	theta = atan(tan(nu+degrad(o))*cos(degrad(i))) + degrad(O);
+	rho = r*cos(nu+degrad(o))/cos(theta-degrad(O));
+	if (rho < 0) {
+	    theta += PI;
+	    rho = -rho;
+	}
+	range (&theta, 2*PI);
+
+	*thetap = theta;
+	*rhop = rho;
+}
+
+/* solve kepler equation using Newton-Raphson search.
+ * Charles and Tatum have shown it always converges starting with PI.
+ */
+static double
+solveKepler (double M, double e)
+{
+	double E, Eprime = PI;
+
+	do {
+	    double cosE = cos(Eprime);
+	    E = Eprime;
+	    Eprime = (M - e*(E*cosE - sin(E)))/(1.0 - e*cosE);
+	} while (fabs(E-Eprime) > 1e-7);
+
+	return (Eprime);
+}
+
+static int
+obj_fixed (Now *np, Obj *op)
 {
 	double lsn, rsn;	/* true geoc lng of sun, dist from sn to earth*/
 	double lam, bet;	/* geocentric ecliptic long and lat */
 	double ha;		/* local hour angle */
 	double el;		/* elongation */
 	double alt, az;		/* current alt, az */
-	double ra, dec;		/* ra and dec at epoch of date */
+	double ra, dec;		/* ra and dec at equinox of date */
+	double rpm, dpm; 	/* astrometric ra and dec with PM to now */
 	double lst;
 
-	if (epoch != EOD && (float)epoch != op->f_epoch) {
-	    /* want a certain epoch -- if it's not what the database is at
-	     * we change the original to save time next time assuming the
-	     * user is likely to stick with this for a while.
-	     */
-	    double tra = op->f_RA, tdec = op->f_dec;
-	    float tepoch = (float)epoch;	/* compare w/float precision */
-	    precess (op->f_epoch, tepoch, &tra, &tdec);
-	    op->f_epoch = tepoch;
-	    op->f_RA = (float)tra;
-	    op->f_dec = (float)tdec;
+	/* on the assumption that the user will stick with their chosen display
+	 * epoch for a while, we move the defining values to match and avoid
+	 * precession for every call until it is changed again.
+	 * N.B. only compare and store jd's to lowest precission (f_epoch).
+	 * N.B. maintaining J2k ref (which is arbitrary) helps avoid accum err
+	 */
+	if (epoch != EOD && (float)epoch != (float)op->f_epoch) {
+	    double pr = op->f_RA, pd = op->f_dec, fe = (float)epoch;
+	    /* first bring back to 2k */
+	    precess (op->f_epoch, J2000, &pr, &pd);
+	    pr += op->f_pmRA*(J2000-op->f_epoch);
+	    pd += op->f_pmdec*(J2000-op->f_epoch);
+	    /* then to epoch */
+	    pr += op->f_pmRA*(fe-J2000);
+	    pd += op->f_pmdec*(fe-J2000);
+	    precess (J2000, fe, &pr, &pd);
+	    op->f_RA = (float)pr;
+	    op->f_dec = (float)pd;
+	    op->f_epoch = (float)fe;
 	}
 
-	/* set ra/dec to astrometric @ epoch of date */
-	ra = op->f_RA;
-	dec = op->f_dec;
-	precess (op->f_epoch, mjd, &ra, &dec);
+	/* apply proper motion .. assume pm epoch reference equals equinox */
+	rpm = op->f_RA + op->f_pmRA*(mjd-op->f_epoch);
+	dpm = op->f_dec + op->f_pmdec*(mjd-op->f_epoch);
+
+	/* set ra/dec to astrometric @ equinox of date */
+	ra = rpm;
+	dec = dpm;
+	precess (op->f_epoch, mjed, &ra, &dec);
 
 	/* convert equatoreal ra/dec to mean geocentric ecliptic lat/long */
-	eq_ecl (mjd, ra, dec, &bet, &lam);
+	eq_ecl (mjed, ra, dec, &bet, &lam);
 
 	/* find solar ecliptical long.(mean equinox) and distance from earth */
 	sunpos (mjed, &lsn, &rsn, NULL);
 
 	/* allow for relativistic light bending near the sun */
-	deflect (mjd, lam, bet, lsn, rsn, 1e10, &ra, &dec);
+	deflect (mjed, lam, bet, lsn, rsn, 1e10, &ra, &dec);
 
 	/* TODO: correction for annual parallax would go here */
 
 	/* correct EOD equatoreal for nutation/aberation to form apparent 
 	 * geocentric
 	 */
-	nut_eq(mjd, &ra, &dec);
-	ab_eq(mjd, lsn, &ra, &dec);
+	nut_eq(mjed, &ra, &dec);
+	ab_eq(mjed, lsn, &ra, &dec);
 	op->s_gaera = (float)ra;
 	op->s_gaedec = (float)dec;
 
@@ -155,10 +271,10 @@ Obj *op;
 	    op->s_dec = (float)dec;
 	} else {
 	    /* annual parallax at time mjd is to be added here, too, but
-	     * technically in the frame of epoch (usually different from mjd)
+	     * technically in the frame of equinox (usually different from mjd)
 	     */
-	    op->s_ra = op->f_RA;	/* already precessed */
-	    op->s_dec = op->f_dec;
+	    op->s_ra = rpm;
+	    op->s_dec = dpm;
 	}
 
 	/* compute elongation from ecliptic long/lat and sun geocentric long */
@@ -185,15 +301,12 @@ Obj *op;
 /* compute sky circumstances of an object in heliocentric elliptic orbit at *np.
  */
 static int
-obj_elliptical (np, op)
-Now *np;
-Obj *op;
+obj_elliptical (Now *np, Obj *op)
 {
 	double lsn, rsn;	/* true geoc lng of sun; dist from sn to earth*/
 	double dt;		/* light travel time to object */
 	double lg;		/* helio long of earth */
-	double nu, ea;		/* true anomaly and eccentric anomaly */
-	double ma;		/* mean anomaly */
+	double nu;		/* true anomaly */
 	double rp=0;		/* distance from the sun */
 	double lo, slo, clo;	/* angle from ascending node */
 	double inc;		/* inclination */
@@ -205,10 +318,10 @@ Obj *op;
 	double Om;		/* long of ascending node. */
 	double lam;    		/* geocentric ecliptic longitude */
 	double bet;    		/* geocentric ecliptic latitude */
-	double e;		/* fast eccentricity */
 	double ll=0, sll, cll;	/* helio angle between object and earth */
 	double mag;		/* magnitude */
 	double e_n;		/* mean daily motion */
+	double tp;		/* time from perihelion (days) */
 	double rpd=0;
 	double y;
 	int pass;
@@ -217,9 +330,6 @@ Obj *op;
 	sunpos (mjed, &lsn, &rsn, 0);
 	lg = lsn + PI;
 
-	/* faster access to eccentricty */
-	e = op->e_e;
-
 	/* mean daily motion is derived fro mean distance */
 	e_n = 0.9856076686/pow((double)op->e_a, 1.5);
 
@@ -234,9 +344,10 @@ Obj *op;
 					degrad (op->e_om), degrad (op->e_Om),
 					&inc, &om, &Om);
 
-	    ma = degrad (op->e_M + (mjed - op->e_cepoch - dt) * e_n);
-	    anomaly (ma, e, &nu, &ea);
-	    rp = op->e_a * (1-e*e) / (1+e*cos(nu));
+	    tp = mjed - dt - (op->e_cepoch - op->e_M/e_n);
+	    if (vrc (&nu, &rp, tp, op->e_e, op->e_a*(1-op->e_e)) < 0)
+		op->o_flags |= NOCIRCUM;
+	    nu = degrad(nu);
 	    lo = nu + om;
 	    slo = sin(lo);
 	    clo = cos(lo);
@@ -293,15 +404,12 @@ Obj *op;
 /* compute sky circumstances of an object in heliocentric hyperbolic orbit.
  */
 static int
-obj_hyperbolic (np, op)
-Now *np;
-Obj *op;
+obj_hyperbolic (Now *np, Obj *op)
 {
 	double lsn, rsn;	/* true geoc lng of sun; dist from sn to earth*/
 	double dt;		/* light travel time to object */
 	double lg;		/* helio long of earth */
-	double nu, ea;		/* true anomaly and eccentric anomaly */
-	double ma;		/* mean anomaly */
+	double nu;		/* true anomaly and eccentric anomaly */
 	double rp=0;		/* distance from the sun */
 	double lo, slo, clo;	/* angle from ascending node */
 	double inc;		/* inclination */
@@ -315,9 +423,9 @@ Obj *op;
 	double bet;    		/* geocentric ecliptic latitude */
 	double e;		/* fast eccentricity */
 	double ll=0, sll, cll;	/* helio angle between object and earth */
-	double n;		/* mean daily motion */
 	double mag;		/* magnitude */
 	double a;		/* mean distance */
+	double tp;		/* time from perihelion (days) */
 	double rpd=0;
 	double y;
 	int pass;
@@ -328,7 +436,6 @@ Obj *op;
 	lg = lsn + PI;
 	e = op->h_e;
 	a = op->h_qp/(e - 1.0);
-	n = .98563/sqrt(a*a*a);
 
 	/* correct for light time by computing position at time mjd, then
 	 *   again at mjd-dt, where
@@ -341,9 +448,10 @@ Obj *op;
 			    degrad (op->h_om), degrad (op->h_Om),
 			    &inc, &om, &Om);
 
-	    ma = degrad ((mjed - op->h_ep - dt) * n);
-	    anomaly (ma, e, &nu, &ea);
-	    rp = a * (e*e-1.0) / (1.0+e*cos(nu));
+	    tp = mjed - dt - op->h_ep;
+	    if (vrc (&nu, &rp, tp, op->h_e, op->h_qp) < 0)
+		op->o_flags |= NOCIRCUM;
+	    nu = degrad(nu);
 	    lo = nu + om;
 	    slo = sin(lo);
 	    clo = cos(lo);
@@ -387,9 +495,7 @@ Obj *op;
 /* compute sky circumstances of an object in heliocentric hyperbolic orbit.
  */
 static int
-obj_parabolic (np, op)
-Now *np;
-Obj *op;
+obj_parabolic (Now *np, Obj *op)
 {
 	double lsn, rsn;	/* true geoc lng of sun; dist from sn to earth*/
 	double lam;    		/* geocentric ecliptic longitude */
@@ -427,9 +533,7 @@ Obj *op;
 /* find sun's circumstances now.
  */
 static int
-sun_cir (np, op)
-Now *np;
-Obj *op;
+sun_cir (Now *np, Obj *op)
 {
 	double lsn, rsn;	/* true geoc lng of sun; dist from sn to earth*/
 	double bsn;		/* true latitude beta of sun */
@@ -457,9 +561,7 @@ Obj *op;
 /* find moon's circumstances now.
  */
 static int
-moon_cir (np, op)
-Now *np;
-Obj *op;
+moon_cir (Now *np, Obj *op)
 {
 	double lsn, rsn;	/* true geoc lng of sun; dist from sn to earth*/
 	double lam;    		/* geocentric ecliptic longitude */
@@ -485,7 +587,9 @@ Obj *op;
 						    - 2.0*edistau*rsn*cos(el));
 
 	/* TODO: improve mag; this is based on a flat moon model. */
-	set_smag (op, -12.7 + 2.5*(log10(PI) - log10(PI/2*(1+1.e-6-cos(el)))));
+	i = -12.7 + 2.5*(log10(PI) - log10(PI/2*(1+1.e-6-cos(el)))) 
+					+ 5*log10(edistau/.0025) /* dist */;
+	set_smag (op, i);
 
 	/* find phase -- allow for projection effects */
 	i = 0.1468*sin(el)*(1 - 0.0549*sin(md))/(1 - 0.0167*sin(ms));
@@ -505,14 +609,17 @@ Obj *op;
  * this is used for sol system objects (except sun and moon); never FIXED.
  */
 static void
-cir_sky (np, lpd, psi, rp, rho, lam, bet, lsn, rsn, op)
-Now *np;
-double lpd, psi;	/* heliocentric ecliptic long and lat */
-double rp;		/* dist from sun */
-double *rho;		/* dist from earth: in as geo, back as geo or topo */
-double lam, bet;	/* true geocentric ecliptic long and lat */
-double lsn, rsn;	/* true geoc lng of sun; dist from sn to earth*/
-Obj *op;
+cir_sky (
+Now *np,
+double lpd,		/* heliocentric ecliptic longitude */
+double psi,		/* heliocentric ecliptic lat */
+double rp,		/* dist from sun */
+double *rho,		/* dist from earth: in as geo, back as geo or topo */
+double lam,		/* true geocentric ecliptic long */
+double bet,		/* true geocentric ecliptic lat */
+double lsn,		/* true geoc lng of sun */
+double rsn,		/* dist from sn to earth*/
+Obj *op)
 {
 	double el;		/* elongation */
 	double f;		/* fractional phase from earth */
@@ -551,7 +658,7 @@ Obj *op;
  *   hadec_aa	--> alt/az	topocentric horizontal
  *   refract	--> alt/az	observed --> output
  *
- * algorithm at fixed epoch:
+ * algorithm at fixed equinox:
  *   ecl_eq	--> ra/dec	geocentric mean equatoreal EOD (via mean obliq)
  *   deflect	--> ra/dec	  relativistic deflection [for alt/az only]
  *   nut_eq	--> ra/dec	geocentric true equatoreal EOD [for aa only]
@@ -563,11 +670,12 @@ Obj *op;
  *   refract	--> alt/az	observed --> output
  */
 static void
-cir_pos (np, bet, lam, rho, op)
-Now *np;
-double bet, lam;/* geo lat/long (mean ecliptic of date) */
-double *rho;	/* in: geocentric dist in AU; out: geo- or topocentic dist */
-Obj *op;	/* object to set s_ra/dec as per epoch */
+cir_pos (
+Now *np,
+double bet,	/* geo lat (mean ecliptic of date) */
+double lam,	/* geo long (mean ecliptic of date) */
+double *rho,	/* in: geocentric dist in AU; out: geo- or topocentic dist */
+Obj *op)	/* object to set s_ra/dec as per equinox */
 {
 	double ra, dec;		/* apparent ra/dec, corrected for nut/ab */
 	double tra, tdec;	/* astrometric ra/dec, no nut/ab */
@@ -580,7 +688,7 @@ Obj *op;	/* object to set s_ra/dec as per epoch */
 	double rho_topo;        /* topocentric distance in earth radii */
 
 	/* convert to equatoreal [mean equator, with mean obliquity] */
-	ecl_eq (mjd, bet, lam, &ra, &dec);
+	ecl_eq (mjed, bet, lam, &ra, &dec);
 	tra = ra;	/* keep mean coordinates */
 	tdec = dec;
 
@@ -591,12 +699,12 @@ Obj *op;	/* object to set s_ra/dec as per epoch */
 	 * (avoid calling deflect() for the sun itself).
 	 */
 	if (!is_planet(op,SUN) && !is_planet(op,MOON))
-	    deflect (mjd, op->s_hlong, op->s_hlat, lsn, rsn, *rho, &ra, &dec);
+	    deflect (mjed, op->s_hlong, op->s_hlat, lsn, rsn, *rho, &ra, &dec);
 
 	/* correct ra/dec to form geocentric apparent */
-	nut_eq (mjd, &ra, &dec);
+	nut_eq (mjed, &ra, &dec);
 	if (!is_planet(op,MOON))
-	    ab_eq (mjd, lsn, &ra, &dec);
+	    ab_eq (mjed, lsn, &ra, &dec);
 	op->s_gaera = (float)ra;
 	op->s_gaedec = (float)dec;
 
@@ -634,7 +742,7 @@ Obj *op;	/* object to set s_ra/dec as per epoch */
 	} else {			/* astrometric geo/topocent */
 	    ra = tra + dra;
 	    dec = tdec + ddec;
-	    precess (mjd, epoch, &ra, &dec);
+	    precess (mjed, epoch, &ra, &dec);
 	}
 	range(&ra, 2*PI);
 	op->s_ra = (float)ra;
@@ -654,9 +762,7 @@ Obj *op;	/* object to set s_ra/dec as per epoch */
  * all angles in radians.
  */
 static void
-elongation (lam, bet, lsn, el)
-double lam, bet, lsn;
-double *el;
+elongation (double lam, double bet, double lsn, double *el)
 {
 	*el = acos(cos(bet)*cos(lam-lsn));
 	if (lam>lsn+PI || (lam>lsn-PI && lam<lsn)) *el = - *el;
@@ -680,12 +786,12 @@ double *el;
  * not the "inertial" J2000 frame.
  */
 static void
-deflect (mjd1, lpd, psi, lsn, rsn, rho, ra, dec)
-double mjd1;		/* epoch */
-double lpd, psi;	/* heliocentric ecliptical long / lat */
-double rsn, lsn;	/* distance and longitude of sun */
-double rho;		/* geocentric distance */
-double *ra, *dec;	/* geocentric equatoreal */
+deflect (
+double mjd1,		/* equinox */
+double lpd, double psi,	/* heliocentric ecliptical long / lat */
+double rsn, double lsn,	/* distance and longitude of sun */
+double rho,		/* geocentric distance */
+double *ra, double *dec)/* geocentric equatoreal */
 {
 	double hra, hdec;	/* object heliocentric equatoreal */
 	double el;		/* HELIOCENTRIC elongation object--earth */
@@ -701,8 +807,9 @@ double *ra, *dec;	/* geocentric equatoreal */
 
 	elongation(lpd, psi, lsn-PI, &el);
 	el = fabs(el);
-	/* only continue if object is within about 10 deg around the sun
-	 * and not obscured by the sun's disc (radius 0.25 deg)
+	/* only continue if object is within about 10 deg around the sun,
+	 * not obscured by the sun's disc (radius 0.25 deg) and farther away
+	 * than the sun.
 	 *
 	 * precise geocentric deflection is:  g1 * tan(el/2)
 	 *	radially outwards from sun;  the vector munching below
@@ -711,7 +818,7 @@ double *ra, *dec;	/* geocentric equatoreal */
 	 *		g1 is always about 0.004 arc seconds
 	 *		g2 varies from 0 (highest contribution) to 2
 	 */
-	if (el<degrad(170) || el>degrad(179.75)) return;
+	if (el<degrad(170) || el>degrad(179.75) || rho<rsn) return;
 
 	/* get cartesian vectors */
 	sphcart(*ra, *dec, rho, u, u+1, u+2);
@@ -747,11 +854,10 @@ double *ra, *dec;	/* geocentric equatoreal */
  * has an absolute mag of 18.
  */
 static double
-h_albsize (H)
-double H;
+h_albsize (double H)
 {
 	return (3600*raddeg(.707*1500*pow(2.51,(18-H)/2)/MAU));
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: circum.c,v $ $Date: 2000/10/07 05:12:17 $ $Revision: 1.4 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: circum.c,v $ $Date: 2004/11/25 20:49:44 $ $Revision: 1.18 $ $Name:  $"};
